#!/bin/env perl

use strict;
use File::Basename qw ( dirname );
use File::Find qw ( find );
use feature qw(say);

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::DataCache::API::Cache qw ( GetCache);
use DE_EPAGES::Database::API::Config qw( StoreType GetBusinessUnits StoreTypesWithCartridge );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory  qw ( LoadRootObject );
use DE_EPAGES::Presentation::API::Template;
use DE_EPAGES::WebInterface::API::MessageCenter qw( GetMessageCenterForScripts );

sub Main {
    local $| = 1;

    chdir dirname $0;

    my $SourceDir = '../FileChanges/srv/epages/eproot/Cartridges/DE_EPAGES';
    return unless -d $SourceDir;
    chdir $SourceDir;

    my ($Stores, $TemplateFiles) = _GetTemplates();

    foreach my $Store (@$Stores) {
        RunOnStore(
            Store => $Store,
            Sub => sub  {
                next unless $Store eq LoadRootObject()->get('TemplateStore');
                my @Lang;
                foreach (@{LoadRootObject()->get('Languages')}) {
                    my $Code = $_->{Code2};
                    $Code .= '-'. $_->{Dialect} if $_->{Dialect};
                    push @Lang, $Code;
                }
                my $IncHandler = DE_EPAGES::Presentation::API::Template->new(force => 1);
                say qq(_CreateCtmpls($Store, [@Lang]));
                _CreateCtmpls($IncHandler, $TemplateFiles->{$Store}, \@Lang);

                # send a cache update message to all application servers,
                # because the TemplateStore could be used by application
                # servers in different pools
                eval {
                    my $MC = GetMessageCenterForScripts();
                    say qq(\$MC->broadcastUpdateCacheRequest($Store));
                    $MC->broadcastUpdateCacheRequest($Store, {TemplateByteCode => 1});
                };
                Warning(GetError()) if ExistsError();
            },
        );
    }
}

sub _CreateCtmpls {
    my ($IncludeHandler, $aTemplateFiles, $aLanguageCodes) = @_;
    return unless scalar @$aTemplateFiles;

    my $TemplateStore = LoadRootObject()->get('TemplateStore');
    my $CacheRef = GetCache($TemplateStore, 'TemplateByteCode');
    foreach my $FileName (@$aTemplateFiles) {
        foreach my $LanguageCode (@$aLanguageCodes) {
            $IncludeHandler->byteCodeByFileName($FileName, $LanguageCode);
            delete $CacheRef->{$IncludeHandler->perlFileName($FileName, $LanguageCode)};
        }
    }
}

sub _GetTemplates {
    my $BU;
    push @{$BU->{StoreType($_)}}, $_ foreach @{GetBusinessUnits({conf => 1})};

    my ($Stores, $TemplateFiles) = ([], {});
    foreach my $Dir (sort glob '*/Templates') {
        my $Cartridge = $Dir;
        $Cartridge =~ s,/.*,,;
        my @StoreTypes = @{StoreTypesWithCartridge("DE_EPAGES::$Cartridge")};

        find(
            sub{
                return unless -f;
                my $FN = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/$File::Find::name";
                foreach my $StoreType (@StoreTypes) {
                    foreach my $Store (@{$BU->{$StoreType}}) {
                        push @$Stores, $Store unless $Store ~~ @$Stores;
                        push @{$TemplateFiles->{$Store}}, $FN;
                    }
                }
            }, $Dir
        );
    }
    return ($Stores, $TemplateFiles);
}

RunScript(
    'Sub' => \&Main
);
