#!/bin/env perl

use strict;
use Config::IniFiles;
use File::Basename qw ( dirname );
use File::Compare  qw ( compare );
use feature qw(say);
use lib qw(. Install);
use Hotfix qw(GetBasicStoreTypes);

use DE_EPAGES::Core::API::File          qw( CopyFile GetFileContentLines WriteFile );
use DE_EPAGES::Core::API::Script        qw( RunScript );
use DE_EPAGES::Installer::API::Service  qw( StartService StopService );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw(GetCartridges);

RunScript(
    'Sub' => \&Main
);

sub Main {
    local $| = 1;

    chdir dirname $0;

    my $SourceDir = 'Scheduler';
    return unless -d $SourceDir;
    chdir $SourceDir;

    my %Cartridges;
    foreach my $StoreType (sort keys %{GetBasicStoreTypes()}) {
        foreach my $Cartridge (@{GetCartridges()->{$StoreType}}) {
            $Cartridge =~ s,DE_EPAGES::,, or next;
            $Cartridges{"$Cartridge.hash"} = 1;
        }
    }

    CopyFile("$ENV{EPAGES_CONFIG}/Scheduler.conf", 'Scheduler.conf.bak');
    foreach (sort glob '*.hash') {
        # update scheduler only for installed cartridges
        next unless defined $Cartridges{$_};
        UpdateConf("$ENV{EPAGES_CONFIG}/Scheduler.conf", $_);
    }

    if (compare("$ENV{EPAGES_CONFIG}/Scheduler.conf", 'Scheduler.conf.bak') != 0) {
        # restart scheduler if configuration has changed
        StopService('cron');
        StartService('cron');
    }
}

#========================================================================================
# function     UpdateConf
#----------------------------------------------------------------------------------------
# syntax       UpdateConf($TargetConf, $ChangesHash);
#----------------------------------------------------------------------------------------
# example      UpdateConf("$ENV{EPAGES_CONFIG}/Scheduler.conf", "Amazon.hash");
#----------------------------------------------------------------------------------------
# description  Updates $TargetConf with information from $ChangesHash. $ChangesHash was
#               created by 'tie $Changes, 'Config::IniFiles', ( -file => ...);'
#               and written with 'print Data::Dumper->Dump([$Changes], ['Changes']);'
#----------------------------------------------------------------------------------------
# input        $TargetConf | name of ini file | string
# input        $ChangesHash | name of file with $Changes hash | string
#========================================================================================
our $Changes; ##no critic (ProhibitPackageVars)
sub UpdateConf ($$) {
    my ($TargetConf, $ChangesHash) = @_;

    say qq(UpdateConf($TargetConf, $ChangesHash));

    require $ChangesHash or die;

    my %Target;
    tie %Target, 'Config::IniFiles', ( -file => $TargetConf ) or die;

    foreach my $Section (keys %$Changes) {

        if (not defined $Changes->{$Section}) {
            # delete section in $Target
            delete $Target{$Section};
        } elsif (not defined $Target{$Section}) {
            # add section in $Target
            $Target{$Section} = { %{$Changes->{$Section}} };
        } else {
            # edit section in $Target
            foreach my $Key (keys %{$Changes->{$Section}}) {
                if (not defined $Changes->{$Section}{$Key}) {
                    # delete key in $Target
                    delete $Target{$Section}{$Key};
                } else {
                    # add/edit key in $Target
                    $Target{$Section}{$Key} = $Changes->{$Section}{$Key};
                }
            }
        }
    }

    # memorize empty values (Ini::Config can't handle them)

    foreach my $Section (keys %Target) {
        foreach (keys %{$Target{$Section}}) {
            $Target{$Section}{$_} = $Target{$Section}{$_}->[0]
                if ref $Target{$Section}{$_} eq 'ARRAY';
            $Target{$Section}{$_} = 'embdiy' if $Target{$Section}{$_} eq '';
        }
    }

    # write config file

    tied(%Target)->RewriteConfig() or die;

    # write empty values (Ini::Config can't handle them)

    my $ContentLines = GetFileContentLines($TargetConf);
    s/=embdiy(\n?)$/=$1/ foreach @$ContentLines;
    my $Content = join('', @$ContentLines);
    WriteFile($TargetConf,\$Content);

    return;
}

1;
