#!/bin/env perl

use strict;
use Getopt::Long;
use File::Basename qw(dirname);
use Sort::Versions qw(versioncmp);

use DE_EPAGES::Core::API::File qw ( GetFileContent GetFileContentLines WriteFile CopyFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (SystemCommand);

sub usage {
    die <<END_USAGE;
Usage:
    perl $0 [-nocheck|-check]

Options:
    -check  :   only run perl scripts starting with '[0-9]+-Check'
    -nocheck: do not run perl scripts starting with '[0-9]+-Check'
END_USAGE
}

sub Main {
    local $| = 1;

    my %Options;
    my @GetOpt = qw(check! final);
    usage() unless GetOptions(\%Options, @GetOpt);
    $Options{check} //= -1;

    chdir dirname $0;

    my $HotfixVersion = GetFileContent('../version.txt');
    chomp $$HotfixVersion;

    my $VersionFile = "$ENV{EPAGES_CONFIG}/hotfixes/version.txt";
    my $Run = 'all';
    if ( -f $VersionFile ) {
        my $InstalledVersions = GetFileContent($VersionFile);
        $Run = 'local' if $$InstalledVersions =~ /(^|\n)\Q$$HotfixVersion\E(\n|$)/;
    }

    my @PostInstall;
    my $PerlExe = $^O eq 'MSWin32' ? $^X : "$ENV{PERL}";
    foreach my $PerlScript (sort glob '[0-9]*.PL') {
        next if $PerlScript =~ /^\d+-Check/ and $Options{check} == 0;
        next if $PerlScript !~ /^\d+-Check/ and $Options{check} == 1;
        next if $Run ne 'local' and $PerlScript =~ /-local\.PL/;
        if ($Options{final}) {
            next if $PerlScript !~ /-final\.PL/;
        } elsif ($Run eq 'local') {
            next if $PerlScript =~ /-(shared|final)\.PL/;
        }
        SystemCommand( "$PerlExe $PerlScript", {Print => 1} );
        die $! if $!;
        @PostInstall = ($$HotfixVersion) if $Run ne 'local' and $PerlScript =~ /-final\.PL/;;
    }

    return if $Run eq 'local';

    my $Dir = $ENV{EPAGES_PATCHES} // "$ENV{EPAGES_SHARED}/Patches";
    CopyFile('postinstall.PL', "$Dir/HotfixPostInstall.PL",
             {create_directory => 1}) if -f 'postinstall.PL';

    return unless scalar @PostInstall;

    my $PostInstallFile = "$ENV{EPAGES_CONFIG}/hotfixes/postinstall.txt";
    if ( -f $PostInstallFile ) {
        foreach (@{GetFileContentLines($PostInstallFile)}) {
            /^([0-9].+)/ or next;
            push @PostInstall, $1 unless $1 ~~ @PostInstall;
        }
    }
    my $Content = join("\n", sort versioncmp @PostInstall);
    WriteFile($PostInstallFile, \$Content);
}

RunScript(
    'Sub' => \&Main
);
